Index: sys/arm/arm/locore.S
===================================================================
--- sys/arm/arm/locore.S	(revision 256775)
+++ sys/arm/arm/locore.S	(working copy)
@@ -175,6 +175,18 @@
 	mcr	p15, 0, r0, c13, c0, 1	/* Set ASID to 0 */
 #endif
 
+#if defined(CPU_CORTEXA)
+	/* TODO: Better to do access to these counters as some kind of device?
+	 * This is 32 bits. Use interrupt to make 64 bits? Random number generator
+	 * doesn't care, but others might. See get_cyclecount().
+	 */
+
+	mov	r7, #0x8000000F /* Set INTENS to 0 to block interrupts */
+	mcr	p15, 0, r7, c9, c14, 2
+	mov	r7, #1		/* Set PMNC[0] to 1 to enable CCNT */
+	mcr	p15, 0, r7, c9, c12, 0
+#endif
+
 	/* Set the Domain Access register.  Very important! */
 	mov     r0, #((DOMAIN_CLIENT << (PMAP_DOMAIN_KERNEL*2)) | DOMAIN_CLIENT)
 	mcr	p15, 0, r0, c3, c0, 0
Index: sys/arm/include/cpu.h
===================================================================
--- sys/arm/include/cpu.h	(revision 256775)
+++ sys/arm/include/cpu.h	(working copy)
@@ -13,11 +13,21 @@
 static __inline uint64_t
 get_cyclecount(void)
 {
+#if defined(CPU_CORTEXA)
+	uint32_t ccnt;
+
+	/*
+	 * Read CCNT. Curses! Its only 32 bits.
+	 * Fix this catching overflow with interrupt in locore.S?
+	 */
+	__asm __volatile("mrc p15, 0, %0, c9, c13, 0": "=r" (ccnt));
+	return ((uint64_t)ccnt);
+#else
 	struct bintime bt;
 
 	binuptime(&bt);
 	return ((uint64_t)bt.sec << 56 | bt.frac >> 8);
-			
+#endif
 }
 #endif
 
